#!/usr/bin/env python3

import sys
import random

class Node:
    def __init__(self, val):
        self.data = val
        self.left  = None
        self.right = None

# The tree CANNOT contain duplicate values
class BinaryTree:
    def __init__(self):
        self.root = None

    def insertNode(self, val):
        self.root = self._insertNode(self.root, val)

    def _insertNode(self, node, val):
        if node is None:
            return Node(val)

        if val == node.data:
            return node

        # This is where we prevent duplicate values
        # No == case is considered
        if val < node.data:
            node.left = self._insertNode(node.left, val)
        elif val > node.data:
            node.right = self._insertNode(node.right, val)
        return node

    # Remove the minimum value from a tree or subtree
    def _remove_min(self, node):
        if node.left is None:
            return node.right

        node.left = self._remove_min(node.left)
        return node

    # Remove value from binary tree
    def remove(self, val):
        self.root = self._remove(self.root, val)

    def _remove(self, node, val):
        if node is None:
            return None

        if val < node.data:
            node.left = self._remove(node.left, val)
        elif val > node.data:
            node.right = self._remove(node.right, val)
        else:
            if node.left is None:
                return node.right
            if node.right is None:
                return node.left

            # replace self value with largest value from left subtree
            original = node

            # find SMALLEST child in right subtree and remove it
            node = node.right
            while node.left:
                node = node.left

            node.right = self._remove_min(original.right)
            node.left = original.left

        return node

    def traverse(self):
        self._traverse(self.root)
        print("")

    def _traverse(self, node):
        if node is None:
            return

        print(node.data, end=" ")
        self._traverse(node.left)
        self._traverse(node.right)

    def height(self):
        return self._height(self.root)

    def _height(self, node):
        if node is None:
            return 0
    
        leftHeight = self._height(node.left)
        rightHeigh = self._height(node.right)

        if (leftHeight > rightHeigh):
            return leftHeight + 1
        else :
            return rightHeigh + 1

    def isBalanced(self):
        return self._isBalanced(self.root)

    def _isBalanced(self, root):
        if root is None:
            return True

        lh = self._height(self.root.left)
        rh = self._height(self.root.right)

        return ((abs(lh - rh) <= 1)
            and self._isBalanced(root.left)
            and self._isBalanced(root.right))

if len(sys.argv) != 2:
    print("./balanced.py NUMBER_OF_NODES")
    sys.exit()

nodes = int(sys.argv[1])
min = 0
max = nodes

myArray = []
for x in range(min, max):
    myArray.append(x)
random.shuffle(myArray)

bt = BinaryTree()
for myInt in myArray:
    bt.insertNode(myInt)

print("Height:", bt.height())
print("Balanced:", bt.isBalanced())
