#!/usr/bin/env python3

import sys
import random

class Node:
    def __init__(self, val):
        self.data = val
        self.left  = None
        self.right = None

# The tree CANNOT contain duplicate values
class BinaryTree:
    def __init__(self):
        self.root = None

    def insertNode(self, val):
        self.root = self._insertNode(self.root, val)

    def _insertNode(self, node, val):
        if node is None:
            return Node(val)

        if val == node.data:
            return node

        # This is where we prevent duplicate values
        # No == case is considered
        if val < node.data:
            node.left = self._insertNode(node.left, val)
        elif val > node.data:
            node.right = self._insertNode(node.right, val)
        return node

    # Remove the minimum value from a tree or subtree
    def _removeMin(self, node):
        if node.left is None:
            return node.right

        node.left = self._removeMin(node.left)
        return node

    # Remove value from binary tree
    def remove(self, val):
        self.root = self._remove(self.root, val)

    def _remove(self, node, val):
        if node is None:
            return None

        if val < node.data:
            node.left = self._remove(node.left, val)
        elif val > node.data:
            node.right = self._remove(node.right, val)
        else:
            if node.left is None:
                return node.right
            if node.right is None:
                return node.left

            # replace self value with largest value from left subtree
            original = node
            # find SMALLEST child in right subtree and remove it
            node = node.right
            while node.left:
                node = node.left

            node.right = self._removeMin(original.right)
            node.left = original.left

        return node

    def traverse(self, node):
        self._traverse(node)
        print("")

    def _traverse(self, node):
        if node is None:
            return
        print(node.data, end=" ")
        self._traverse(node.left)
        self._traverse(node.right)

if len(sys.argv) != 3:
    print("./traverse.py NUMBER_OF_NODES REMOVE_VALUE")
    sys.exit()

nodes = int(sys.argv[1])
removeValue = int(sys.argv[2])
min = 0
max = nodes

myArray = []
for x in range(min, max):
    myArray.append(x)

random.shuffle(myArray)

bt = BinaryTree()
for myInt in myArray:
    bt.insertNode(myInt)

bt.traverse(bt.root)
bt.remove(removeValue)
bt.traverse(bt.root)
